@@ -13,6 +13,9 @@ NoLearnAgent
 Agent
     the base class of all agent classes that learn from history
 
+AgentDemon:
+    A class that allows an `agent` to be manipulated.
+
 QLearning
     the Q-learning agent that can accept any learner
 
@@ -27,26 +30,16 @@ UserAgent
     an agent that shows current state and asks for user's choice
     of action
 
-WarfarinAgent
-    an agent based on Ravvaz et al (2016) paper for Warfarin Dosing
-
 Types
 -----
 TrainingData
     a type alias for training data, consisting of a tuple of
-    `ReilData` for X matrix and a tuple of floats for Y vector.
+    `FeatureArray` for X matrix and a tuple of floats for Y vector.
 
-AgentType
-    a type variable that is bound to `Agent`.
 '''
-from .no_learn_agent import NoLearnAgent, TrainingData  # noqa: W0611
-from .agent import Agent  # noqa: W0611
-from .q_learning import QLearning  # noqa: W0611
-from .deep_q_learning import DeepQLearning  # noqa: W0611
-from .random_agent import RandomAgent  # noqa: W0611
-from .user_agent import UserAgent  # noqa: W0611
-from .warfarin_agent import WarfarinAgent  # noqa: W0611
-
-from typing import TypeVar
-
-AgentType = TypeVar('AgentType', bound=Agent)
+from .no_learn_agent import NoLearnAgent  # noqa: W601
+from .agent import Agent, TrainingData  # noqa: W601
+from .q_learning import QLearning  # noqa: W601
+from .deep_q_learning import DeepQLearning  # noqa: W601
+from .random_agent import RandomAgent  # noqa: W601
+from .user_agent import UserAgent  # noqa: W601
