@@ -8,8 +8,9 @@ A Q-learning `agent` with a Neural Network Q-function approximator.
 
 from typing import Any
 
-from reil.agents import QLearning
+from reil.agents.q_learning import QLearning
 from reil.datatypes.buffers import VanillaExperienceReplay
+from reil.datatypes.feature import FeatureArray
 from reil.learners import Dense
 from reil.utils.exploration_strategies import ExplorationStrategy
 
@@ -21,7 +22,7 @@ class DeepQLearning(QLearning):
 
     def __init__(self,
                  learner: Dense,
-                 buffer: VanillaExperienceReplay,
+                 buffer: VanillaExperienceReplay[FeatureArray, float],
                  exploration_strategy: ExplorationStrategy,
                  method: str = 'backward',
                  **kwargs: Any):
@@ -38,7 +39,7 @@ class DeepQLearning(QLearning):
         exploration_strategy:
             An `ExplorationStrategy` object that determines
             whether the `action` should be exploratory or not for a given
-            `state` at a given `epoch`.
+            `state` at a given `iteration`.
 
         method:
             Either 'forward' or 'backward' Q-learning.
