@@ -7,14 +7,11 @@ The base class of all `agent` classes.
 '''
 
 import random
-from typing import Any, Optional, Tuple, TypeVar
+from typing import Any, Generator, Literal, Optional, Tuple, TypeVar, Union
 
 from reil import stateful
-from reil.datatypes.reildata import ReilData
-from reil.utils import functions
-from typing_extensions import Literal
+from reil.datatypes.feature import FeatureArray
 
-TrainingData = Tuple[Tuple[ReilData, ...], Tuple[float, ...]]
 T = TypeVar('T')
 
 
@@ -25,7 +22,7 @@ class NoLearnAgent(stateful.Stateful):
     '''
 
     def __init__(self,
-                 default_actions: Tuple[ReilData, ...] = (),
+                 default_actions: Tuple[FeatureArray, ...] = (),
                  tie_breaker: Literal['first', 'last', 'random'] = 'random',
                  **kwargs: Any):
         '''
@@ -46,17 +43,18 @@ class NoLearnAgent(stateful.Stateful):
         super().__init__(**kwargs)
 
         self._default_actions = default_actions
+        self._training_trigger = 'none'
 
-        self.training_mode = False
         if tie_breaker not in ['first', 'last', 'random']:
             raise ValueError(
                 'Tie breaker should be one of first, last, or random options.')
         self._tie_breaker: Literal['first', 'last', 'random'] = tie_breaker
 
     def act(self,
-            state: ReilData,
-            actions: Optional[Tuple[ReilData, ...]] = None,
-            epoch: int = 0) -> ReilData:
+            state: FeatureArray,
+            subject_id: int,
+            actions: Optional[Tuple[FeatureArray, ...]] = None,
+            iteration: int = 0) -> FeatureArray:
         '''
         Return an action based on the given state.
 
@@ -68,16 +66,15 @@ class NoLearnAgent(stateful.Stateful):
         actions:
             The set of possible actions to choose from.
 
-        epoch:
-            The epoch in which the agent is acting.
+        iteration:
+            The iteration in which the agent is acting.
 
         Returns
         -------
         :
             The action
         '''
-        possible_actions = functions.get_argument(
-            actions, self._default_actions)
+        possible_actions = actions or self._default_actions
 
         result = self.best_actions(state, possible_actions)
 
@@ -89,9 +86,9 @@ class NoLearnAgent(stateful.Stateful):
         return action
 
     def best_actions(self,
-                     state: ReilData,
-                     actions: Tuple[ReilData, ...],
-                     ) -> Tuple[ReilData, ...]:
+                     state: FeatureArray,
+                     actions: Tuple[FeatureArray, ...],
+                     ) -> Tuple[FeatureArray, ...]:
         '''
         Find the best `action`s for the given `state`.
 
@@ -110,6 +107,62 @@ class NoLearnAgent(stateful.Stateful):
         '''
         raise NotImplementedError
 
+    def observe(self, subject_id: int, stat_name: str,
+                ) -> Generator[Union[FeatureArray, None], Any, None]:
+        '''
+        Create a generator to interact with the subject (`subject_id`).
+
+        This method creates a generator for `subject_id` that
+        receives `state`, yields `action` and receives `reward`
+        until it is closed. When `.close()` is called on the generator,
+        `statistics` are calculated.
+
+        Arguments
+        ---------
+        subject_id:
+            the ID of the `subject` on which action happened.
+
+        stat_name:
+            The name of the `statistic` that should be computed at the end of
+            each trajectory.
+
+        Raises
+        ------
+        ValueError
+            Subject with `subject_id` not found.
+        '''
+        if subject_id not in self._entity_list:
+            raise ValueError(f'Subject with ID={subject_id} not found.')
+
+        history: stateful.History = []
+        new_observation = stateful.Observation()
+        while True:
+            try:
+                new_observation = stateful.Observation()
+                temp = yield
+                new_observation.state = temp['state']
+                actions: Tuple[FeatureArray, ...] = temp['actions']
+                iteration: int = temp['iteration']
+
+                if actions is not None:
+                    new_observation.action = self.act(
+                        state=new_observation.state,  # type: ignore
+                        subject_id=subject_id,
+                        actions=actions, iteration=iteration)
+
+                    new_observation.reward = (yield new_observation.action)
+
+                    history.append(new_observation)
+
+            except GeneratorExit:
+                if new_observation.reward is None:  # terminated early!
+                    history.append(new_observation)
+
+                # self._computed_stats.append(
+                #     self.statistic(stat_name, subject_id))
+
+                return
+
     @staticmethod
     def _break_tie(input_tuple: Tuple[T, ...],
                    method: Literal['first', 'last', 'random']) -> T:
