@@ -9,25 +9,25 @@ An agent that randomly chooses an action
 import random
 from typing import Any, Optional, Tuple
 
-from reil import agents
-from reil.datatypes.reildata import ReilData
-from reil.utils import functions
+from reil.agents.no_learn_agent import NoLearnAgent
+from reil.datatypes.feature import FeatureArray
 
 
-class RandomAgent(agents.NoLearnAgent):
+class RandomAgent(NoLearnAgent):
     '''
     An agent that acts randomly.
     '''
 
     def __init__(self,
-                 default_actions: Tuple[ReilData, ...] = (),
+                 default_actions: Tuple[FeatureArray, ...] = (),
                  **kwargs: Any):
         super().__init__(default_actions=default_actions, **kwargs)
 
     def act(self,
-            state: ReilData,
-            actions: Optional[Tuple[ReilData, ...]] = None,
-            epoch: int = 0) -> ReilData:
+            state: FeatureArray,
+            subject_id: int,
+            actions: Optional[Tuple[FeatureArray, ...]] = None,
+            iteration: int = 0) -> FeatureArray:
         '''
         Return a random action.
 
@@ -39,13 +39,12 @@ class RandomAgent(agents.NoLearnAgent):
         actions:
             The set of possible actions to choose from.
 
-        epoch:
-            The epoch in which the agent is acting.
+        iteration:
+            The iteration in which the agent is acting.
 
         Returns
         -------
         :
             The action
         '''
-        return random.choice(functions.get_argument(
-            actions, self._default_actions))
+        return random.choice(actions or self._default_actions)
