@@ -7,25 +7,26 @@ An agent that prints the state and asks the user for action.
 '''
 from typing import Any, Optional, Tuple
 
-from reil import agents
-from reil.datatypes.reildata import ReilData
-from reil.utils import functions
+from reil.agents.no_learn_agent import NoLearnAgent
+from reil.datatypes.feature import FeatureArray
 
 
-class UserAgent(agents.NoLearnAgent):
+class UserAgent(NoLearnAgent):
     '''
     An agent that acts based on user input.
     '''
 
-    def __init__(self,
-                 default_actions: Tuple[ReilData, ...] = (),
-                 **kwargs: Any):
+    def __init__(
+            self,
+            default_actions: Tuple[FeatureArray, ...] = (),
+            **kwargs: Any):
         super().__init__(default_actions=default_actions, **kwargs)
 
     def act(self,
-            state: ReilData,
-            actions: Optional[Tuple[ReilData, ...]] = None,
-            epoch: int = 0) -> ReilData:
+            state: FeatureArray,
+            subject_id: int,
+            actions: Optional[Tuple[FeatureArray, ...]] = None,
+            iteration: int = 0) -> FeatureArray:
         '''
         Return a random action.
 
@@ -37,21 +38,23 @@ class UserAgent(agents.NoLearnAgent):
         actions:
             The set of possible actions to choose from.
 
-        epoch:
-            The epoch in which the agent is acting.
+        iteration:
+            The iteration in which the agent is acting.
 
         Returns
         -------
         :
             The action
         '''
-        possible_actions = functions.get_argument(
-            actions, self._default_actions)
+        possible_actions = actions or self._default_actions
 
         action = None
         while action is None:
             for i, a in enumerate(possible_actions):
-                print(f'{i}. {a.value}')
-            action = input(f'Choose action for this state: {state.value}')
+                print(f'{i}. {a.value}')  # type: ignore
+            action = int(input(
+                'Choose action number for this state:'
+                f'{state.value}')  # type: ignore
+                )
 
         return possible_actions[action]
