@@ -7,19 +7,26 @@ A datatype that accepts initial value and feature generator, and generates
 new values. This datatype uses `Entity` to specify an `agent` or a `subject`.
 '''
 import dataclasses
-
-from typing_extensions import Literal
+from typing import Literal, Optional, Tuple
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class Entity:
     '''
     The datatype to specify an `agent` or a `subject`.
     Used in `InteractionProtocol`.
     '''
     name: str
-    statistic_name: str
-    aggregator_name: str
+    demon_name: Optional[str] = None
+    statistic_name: Optional[str] = None
+    groupby: Optional[Tuple[str, ...]] = None
+    aggregators: Optional[Tuple[str, ...]] = None
+
+    def __post_init__(self):
+        if self.groupby is not None:
+            self.__dict__['groupby'] = tuple(self.groupby)
+        if self.aggregators is not None:
+            self.__dict__['aggregators'] = tuple(self.aggregators)
 
 
 @dataclasses.dataclass
@@ -31,6 +38,7 @@ class InteractionProtocol:
     agent: Entity
     subject: Entity
     state_name: str
-    reward_function_name: str
+    action_name: str
+    reward_name: str
     n: int
-    unit: Literal['interaction', 'instance', 'epoch']
+    unit: Literal['interaction', 'instance', 'iteration']
