@@ -5,11 +5,12 @@ Learner class
 
 The base class for all `learner` classes.
 '''
-import pathlib
-from typing import Any, Generic, Optional, Tuple, TypeVar, Union
+from typing import Any, Generic, Tuple, TypeVar
 
-from reil import learners, reilbase
-from reil.datatypes import ReilData
+from reil import reilbase
+from reil.datatypes.feature import FeatureArray
+from reil.learners.learning_rate_schedulers import (ConstantLearningRate,
+                                                    LearningRateScheduler)
 
 LabelType = TypeVar('LabelType')
 
@@ -19,37 +20,32 @@ class Learner(reilbase.ReilBase, Generic[LabelType]):
     The base class for all `learner` classes.
     '''
     def __init__(self,
-                 learning_rate: learners.LearningRateScheduler,
+                 learning_rate: LearningRateScheduler,
                  **kwargs: Any) -> None:
         '''
         Arguments
         ---------
         learning_rate:
             A `LearningRateScheduler` object that determines the learning rate
-            based on epoch. If any scheduler other than constant is provided,
-            the model uses the `new_rate` method of the scheduler to determine
-            the learning rate at each epoch.
+            based on iteration. If any scheduler other than constant is
+            provided, the model uses the `new_rate` method of the scheduler to
+            determine the learning rate at each iteration.
         '''
         super().__init__(**kwargs)
         self._learning_rate = learning_rate
 
     @classmethod
-    def from_pickle(cls,
-                    filename: str,
-                    path: Optional[Union[pathlib.Path, str]] = None):
-        instance = cls(learning_rate=learners.ConstantLearningRate(0.0))
-        instance.load(filename=filename, path=path)
+    def _empty_instance(cls):
+        return cls(learning_rate=ConstantLearningRate(0.0))
 
-        return instance
-
-    def predict(self, X: Tuple[ReilData, ...]) -> Tuple[LabelType, ...]:
+    def predict(self, X: Tuple[FeatureArray, ...]) -> Tuple[LabelType, ...]:
         '''
         predict `y` for a given input list `X`.
 
         Arguments
         ---------
         X:
-            A list of `ReilData` as inputs to the prediction model.
+            A list of `FeatureArray` as inputs to the prediction model.
 
         Returns
         -------
@@ -58,14 +54,16 @@ class Learner(reilbase.ReilBase, Generic[LabelType]):
         '''
         raise NotImplementedError
 
-    def learn(self, X: Tuple[ReilData, ...], Y: Tuple[LabelType, ...]) -> None:
+    def learn(
+            self, X: Tuple[FeatureArray, ...], Y: Tuple[LabelType, ...]
+    ) -> None:
         '''
         Learn using the training set `X` and `Y`.
 
         Arguments
         ---------
         X:
-            A list of `ReilData` as inputs to the learning model.
+            A list of `FeatureArray` as inputs to the learning model.
 
         Y:
             A list of float labels for the learning model.
