@@ -9,10 +9,11 @@ type of inputs to be `TableEntry`.
 `QLookupTable` is a lookup table for `Q-learning`.
 '''
 import dataclasses
-from typing import Any, Dict, Generic, Tuple, TypeVar
+from typing import Any, Dict, Generic, Hashable, Tuple, TypeVar
 
-from reil import learners
-from reil.datatypes import ReilData
+from reil.learners.learner import Learner
+from reil.datatypes.feature import FeatureArray
+from reil.learners.learning_rate_schedulers import LearningRateScheduler
 
 T = TypeVar('T')
 
@@ -30,23 +31,24 @@ class LookupTable(Dict[Any, TableEntry[T]]):
     A simple lookup table based on `dict` that checks for the type of inputs
     to be `TableEntry`.
     '''
-    def __setitem__(self, key, item):
+    def __setitem__(self, key: Hashable, item: Any):
         if isinstance(item, TableEntry):
             super().__setitem__(key, item)
         else:
             raise TypeError('item should be of type TableEntry.')
 
 
-class QLookupTable(learners.Learner[float]):
+class QLookupTable(Learner[float]):
     '''
     A Q-learning lookup table class.
 
     This class stores input data and the corresponding output as a dictionay.
     '''
-    def __init__(self,
-                 learning_rate: learners.LearningRateScheduler,
-                 initial_estimate: float = 0.0,
-                 minimum_visits: int = 0) -> None:
+    def __init__(
+            self,
+            learning_rate: LearningRateScheduler,
+            initial_estimate: float = 0.0,
+            minimum_visits: int = 0) -> None:
         '''
         Arguments
         ---------
@@ -70,14 +72,14 @@ class QLookupTable(learners.Learner[float]):
         # It creates entries as soon as they are looked up.
         self._table: LookupTable[float] = LookupTable()
 
-    def predict(self, X: Tuple[ReilData, ...]) -> Tuple[float, ...]:
+    def predict(self, X: Tuple[FeatureArray, ...]) -> Tuple[float, ...]:
         '''
         predict `y` for a given input list `X`.
 
         Arguments
         ---------
         X:
-            A list of `ReilData` as inputs to the prediction model.
+            A list of `FeatureArray` as inputs to the prediction model.
 
         Returns
         -------
@@ -93,14 +95,14 @@ class QLookupTable(learners.Learner[float]):
 
         return result
 
-    def learn(self, X: Tuple[ReilData, ...], Y: Tuple[float, ...]) -> None:
+    def learn(self, X: Tuple[FeatureArray, ...], Y: Tuple[float, ...]) -> None:
         '''
         Learn using the training set `X` and `Y`.
 
         Arguments
         ---------
         X:
-            A list of `ReilData` as inputs to the learning model.
+            A list of `FeatureArray` as inputs to the learning model.
 
         Y:
             A list of float labels for the learning model.
@@ -114,7 +116,7 @@ class QLookupTable(learners.Learner[float]):
             self._table[Xi].N += 1
 
     # def load(self, filename: str,
-    #          path: Optional[Union[str, pathlib.Path]] = None) -> None:
+    #          path: Optional[Union[str, pathlib.PurePath]] = None) -> None:
     #     temp = defaultdict(
     #         lambda: {'value': self._initial_estimate,
     #                  'N': 0})
@@ -125,8 +127,8 @@ class QLookupTable(learners.Learner[float]):
 
     # def save(self,
     #          filename: str,
-    #          path: Optional[Union[str, pathlib.Path]] = None
-    #          ) -> Tuple[pathlib.Path, str]:
+    #          path: Optional[Union[str, pathlib.PurePath]] = None
+    #          ) -> Tuple[pathlib.PurePath, str]:
     #     _path = pathlib.Path(path if path is not None else '')
     #     with open(_path / f'{filename}.csv', 'w') as f:
     #         w = csv.writer(f)
