@@ -8,6 +8,8 @@ The base class for reinforcement learning
 
 from __future__ import annotations
 
+import bz2
+import copy
 import importlib
 import logging
 import pathlib
@@ -17,12 +19,10 @@ from typing import Any, Dict, List, Optional, OrderedDict, Tuple, Union
 import dill
 from ruamel.yaml import YAML
 
+import reil
 
-def get_argument(x: Any, y: Any) -> Any:
-    '''
-    :meta private:
-    '''
-    return x if x is not None else y
+
+Parsable = Union[OrderedDict[str, Any], Any]
 
 
 class ReilBase:
@@ -32,11 +32,12 @@ class ReilBase:
 
     def __init__(self,
                  name: Optional[str] = None,
-                 path: Optional[pathlib.Path] = None,
+                 path: Optional[pathlib.PurePath] = None,
                  logger_name: Optional[str] = None,
                  logger_level: Optional[int] = None,
                  logger_filename: Optional[str] = None,
                  persistent_attributes: Optional[List[str]] = None,
+                 save_zipped: bool = False,
                  **kwargs: Any):
         '''
         Arguments
@@ -89,15 +90,16 @@ class ReilBase:
             >>> instance._my_attr
             test
         '''
-        self._name = get_argument(name, __name__.lower())
-        self._path = pathlib.Path(get_argument(path, '.'))
+        self._name = name or self.__class__.__qualname__.lower()
+        self._path = pathlib.PurePath(path or '.')
+        self._save_zipped = save_zipped
 
         self._persistent_attributes = [
             '_' + p
-            for p in get_argument(persistent_attributes, [])]
+            for p in (persistent_attributes or [])]
 
-        self._logger_name = get_argument(logger_name, __name__)
-        self._logger_level = get_argument(logger_level, logging.WARNING)
+        self._logger_name = logger_name or __name__
+        self._logger_level = logger_level or logging.WARNING
         self._logger_filename = logger_filename
 
         self._logger = logging.getLogger(self._logger_name)
@@ -107,10 +109,14 @@ class ReilBase:
 
         self.set_params(**kwargs)
 
+    @classmethod
+    def _empty_instance(cls):
+        return cls()
+
     @classmethod
     def from_pickle(
             cls, filename: str,
-            path: Optional[Union[pathlib.Path, str]] = None) -> ReilBase:
+            path: Optional[Union[pathlib.PurePath, str]] = None):
         '''
         Load a pickled instance.
 
@@ -127,7 +133,7 @@ class ReilBase:
         :
             A `ReilBase` instance.
         '''
-        instance = cls()
+        instance = cls._empty_instance()
         instance._logger_name = __name__
         instance._logger_level = logging.WARNING
         instance._logger_filename = None
@@ -143,7 +149,7 @@ class ReilBase:
     @classmethod
     def from_yaml_file(cls, node_reference: Tuple[str, ...],
                        filename: str,
-                       path: Optional[Union[pathlib.Path, str]] = None):
+                       path: Optional[Union[pathlib.PurePath, str]] = None):
         '''
         Create an instance based on a yaml file.
 
@@ -164,13 +170,13 @@ class ReilBase:
         :
             The generated instance.
         '''
-        _path = pathlib.Path(get_argument(path, '.'))
+        _path = pathlib.Path(path or '.')
         _filename = filename if filename.endswith((
             '.yaml', '.yml')) else f'{filename}.yaml'
 
         yaml = YAML()
         with open(_path / _filename, 'r') as f:
-            yaml_output = yaml.load(f)
+            yaml_output: OrderedDict[str, Any] = yaml.load(f)  # type: ignore
 
         temp_yaml = yaml_output
         for key in node_reference:
@@ -179,7 +185,7 @@ class ReilBase:
         return cls.parse_yaml(temp_yaml)
 
     @staticmethod
-    def parse_yaml(data: OrderedDict):
+    def parse_yaml(data: Parsable) -> Parsable:  # noqa: C901
         '''
         Parse a yaml tree.
 
@@ -204,6 +210,14 @@ class ReilBase:
         if isinstance(data, (int, float, str)):
             return data
 
+        if 'eval' in data:
+            args = {'reil': reil}
+            if 'args' in data:
+                args.update(ReilBase.parse_yaml(data['args']))
+            # else:
+            #     args = {}
+            return eval(data['eval'], args)
+
         if len(data) == 1:
             k, v = next(iter(data.items()))
             result = ReilBase._create_component_from_yaml(k, v)
@@ -215,10 +229,15 @@ class ReilBase:
             if isinstance(v, dict):
                 v_obj = ReilBase.parse_yaml(data[k])
             elif isinstance(v, list):
-                v_obj = [ReilBase.parse_yaml(v_i)
-                         for v_i in v]
-            elif isinstance(v, str) and 'lambda' in v:
-                v_obj = eval(v)
+                v_obj = [
+                    ReilBase.parse_yaml(v_i) for v_i in v]  # type: ignore
+            elif isinstance(v, str):
+                if v.startswith('lambda'):
+                    v_obj = eval(v, {})
+                elif v.startswith('eval'):
+                    v_obj = eval(v[4:], {})
+                else:
+                    v_obj = v
             else:
                 v_obj = v
 
@@ -227,7 +246,7 @@ class ReilBase:
         return args
 
     @staticmethod
-    def _create_component_from_yaml(name: str, args: OrderedDict):
+    def _create_component_from_yaml(name: str, args: OrderedDict[str, Any]):
         '''
         Create a component from yaml data.
 
@@ -276,8 +295,8 @@ class ReilBase:
         for key, value in params.items():
             self.__dict__[f'_{key}'] = value
 
-    def load(self, filename: str,
-             path: Optional[Union[str, pathlib.Path]] = None) -> None:
+    def load(self, filename: str,  # noqa: C901
+             path: Optional[Union[str, pathlib.PurePath]] = None) -> None:
         '''
         Load an object from a file.
 
@@ -297,51 +316,59 @@ class ReilBase:
         RuntimeError
             Corrupted or inaccessible data file.
         '''
-        _path = pathlib.Path(get_argument(path, self._path))
+        _filename = (filename if filename.endswith(('.pkl', '.pbz2'))
+                     else
+                     f'{filename}{".pbz2" if self._save_zipped else ".pkl"}')
+        full_path = pathlib.Path(path or self._path) / _filename
 
-        with open(_path / f'{filename}.pkl', 'rb') as f:
+        data: Optional[Dict[str, Any]] = None
+        for i in range(1, 6):
             try:
-                data = dill.load(f)
-            except EOFError:
-                try:
-                    # self._logger.info(
-                    #     'First attempt failed to load '
-                    #     f'{_path / f"{filename}.pkl"}.')
-                    time.sleep(1)
-                    data = dill.load(f)
-                except EOFError:
-                    # self._logger.exception(
-                    #     'Corrupted or inaccessible data file: '
-                    #     f'{_path / f"{filename}.pkl"}')
-                    raise RuntimeError(
-                        f'Corrupted or inaccessible data file: '
-                        f'{_path / f"{filename}.pkl"}')
+                if self._save_zipped:
+                    with bz2.BZ2File(full_path, 'r') as f:
+                        data = dill.load(f)  # type: ignore
+                else:
+                    with open(full_path, 'rb') as f:
+                        data = dill.load(f)  # type: ignore
+            except FileNotFoundError:
+                raise
+            except (EOFError, OSError):
+                self._logger.info(
+                    f'Attempt {i} failed to load '
+                    f'{full_path}.')
+                time.sleep(2)
+            if data is not None:
+                break
 
-            # self._logger.info(
-            #     'Changing the logger from '
-            #     f'{self._logger_name} to {data["_logger_name"]}.')
+        if data is None:
+            self._logger.exception(
+                'Corrupted or inaccessible data file: '
+                f'{full_path}')
+            raise RuntimeError(
+                f'Corrupted or inaccessible data file: '
+                f'{full_path}')
 
-            persistent_attributes = self._persistent_attributes + \
-                ['_persistent_attributes', 'version']
-            for key, value in data.items():
-                if key not in persistent_attributes:
-                    self.__dict__[key] = value
+        self._logger.info(
+            'Changing the logger from '
+            f'{self._logger_name} to {data["_logger_name"]}.')
 
-            # TODO: classes should use `loaded_version` to compare old vs new
-            # and modify attributes if necessary.
-            self.loaded_version = data.get('version')
+        persistent_attributes = self._persistent_attributes + \
+            ['_persistent_attributes']
+        for key, value in data.items():
+            if key not in persistent_attributes:
+                self.__dict__[key] = value
 
-            self._logger = logging.getLogger(self._logger_name)
-            self._logger.setLevel(self._logger_level)
-            if self._logger_filename is not None:
-                self._logger.addHandler(
-                    logging.FileHandler(self._logger_filename))
+        self._logger = logging.getLogger(self._logger_name)
+        self._logger.setLevel(self._logger_level)
+        if self._logger_filename is not None:
+            self._logger.addHandler(
+                logging.FileHandler(self._logger_filename))
 
     def save(self,
              filename: Optional[str] = None,
-             path: Optional[Union[str, pathlib.Path]] = None,
+             path: Optional[Union[str, pathlib.PurePath]] = None,
              data_to_save: Optional[Tuple[str, ...]] = None
-             ) -> Tuple[pathlib.Path, str]:
+             ) -> Tuple[pathlib.PurePath, str]:
         '''
         Save the object to a file.
 
@@ -355,7 +382,7 @@ class ReilBase:
 
         data_to_save:
             a list of variables that should be pickled. If omitted,
-            the `agent` is saved completely.
+            the object is saved completely.
 
         Returns
         -------
@@ -364,23 +391,31 @@ class ReilBase:
             as `str`
         '''
         if data_to_save is None:
-            data = self.__dict__.copy()
+            data = self.__dict__
         else:
-            data = dict((d, self.__dict__[d])
-                        for d in list(data_to_save) + ['_name', '_path'])
+            data = {d: self.__dict__[d]
+                    for d in list(data_to_save) + ['_name', '_path']}
 
+        temp = None
         if '_logger' in data:
+            temp = copy.deepcopy(self._logger)
             data.pop('_logger')
 
-        _filename: str = get_argument(filename, self._name)
-        _path: pathlib.Path = pathlib.Path(
-            get_argument(path, self._path))
-
+        _filename = filename or self._name
+        _path = pathlib.Path(path or self._path)
         _path.mkdir(parents=True, exist_ok=True)
-        with open(_path / f'{_filename}.pkl', 'wb+') as f:
-            dill.dump(data, f, dill.HIGHEST_PROTOCOL)
 
-        return _path, _filename
+        if self._save_zipped:
+            with bz2.BZ2File(_path / f'{_filename}.pbz2', 'w') as f:
+                dill.dump(data, f, dill.HIGHEST_PROTOCOL)  # type: ignore
+        else:
+            with open(_path / f'{_filename}.pkl', 'wb+') as f:
+                dill.dump(data, f, dill.HIGHEST_PROTOCOL)  # type: ignore
+
+        if temp:
+            self._logger = temp
+
+        return pathlib.PurePath(_path), _filename
 
     def reset(self) -> None:
         ''' Reset the object.'''
