@@ -1,18 +1,18 @@
 # -*- coding: utf-8 -*-
 '''
 TicTacToe class
-==============
+===============
 
 The standard Tic-Tac-Toe game.
 '''
 import random
-from typing import Any, Optional
+from typing import Any, Dict, Optional
 
-from reil import subjects
-from reil.datatypes import reildata
+from reil.subjects.mnkgame import MNKGame
+from reil.datatypes.feature import FeatureArray, FeatureGenerator
 
 
-class TicTacToe(subjects.MNKGame):
+class TicTacToe(MNKGame):
     '''
     Build a 3-by-3 board in which 2 players can play.
     Winner is the player who can put 3 pieces in one row, column, or diagonal.
@@ -25,20 +25,19 @@ class TicTacToe(subjects.MNKGame):
 
     def __init__(self, **kwargs: Any):
         super().__init__(m=3, n=3, k=3, players=2, **kwargs)
+        self._state_gen = FeatureGenerator.numerical(
+            name='state', lower=-1, upper=1)
 
-    def default_state(self, _id: Optional[int] = None) -> reildata.ReilData:
-        def modify(i, _id) -> float:
+    def default_state(self, _id: Optional[int] = None) -> FeatureArray:
+        def modify(i: int, _id: Optional[int]) -> float:
             if i == _id:
                 return 1
             if i == 0:
                 return 0
             return -1
 
-        return reildata.ReilData.single_numerical(
-            name='state',
-            value=tuple(modify(i, _id)
-                        for i in self._board),
-            lower=-1, upper=1)
+        return FeatureArray(self._state_gen(tuple(modify(i, _id)
+                                                  for i in self._board)))
 
     def __repr__(self):
         return self.__class__.__qualname__
@@ -46,7 +45,7 @@ class TicTacToe(subjects.MNKGame):
 
 if __name__ == '__main__':
     board = TicTacToe()
-    player = {}
+    player: Dict[str, int] = {}
     p = 0
     player['P1'] = board.register('P1')
     player['P2'] = board.register('P2')
