@@ -4,14 +4,14 @@ ActionGenerator class
 =====================
 
 Gets lists of categorical or numerical lists as components, and generates lists
-of `ReilData` objects using the product of these components.
+of `FeatureArray` objects using the product of these components.
 '''
 import dataclasses
 import itertools
 from typing import (Any, Dict, Generic, Iterator, Optional, Tuple, TypeVar,
                     Union)
 
-from reil.datatypes import ReilData
+from reil.datatypes.feature import Feature, FeatureArray, FeatureGenerator
 from reil.reilbase import ReilBase
 
 Categorical = TypeVar('Categorical')
@@ -24,18 +24,21 @@ class CategoricalComponent(Generic[Categorical]):
     possible_values: Tuple[Tuple[Categorical, ...], ...]
     categories: Tuple[Categorical, ...]
     length: int = dataclasses.field(init=False, hash=False, compare=False)
+    feature_generator: FeatureGenerator[Categorical] = dataclasses.field(
+        init=False, hash=False, compare=False)
 
     def __post_init__(self):
         object.__setattr__(self, 'length', len(self.possible_values))
+        object.__setattr__(
+            self, 'feature_generator',
+            FeatureGenerator.categorical(
+                name=self.name, categories=self.categories))
 
-    def generate(self, index: int) -> Iterator[Dict[str, Any]]:
+    def generate(self, index: int) -> Iterator[
+            Union[Feature[Categorical], Feature[Tuple[Categorical, ...]]]]:
         _index = min(index, self.length - 1)
-        return (
-            {'name': self.name,
-             'categorical': True,
-             'value': vi,
-             'categories': self.categories}
-            for vi in self.possible_values[_index])
+        return (self.feature_generator(vi)
+                for vi in self.possible_values[_index])
 
 
 @dataclasses.dataclass(frozen=True)
@@ -45,32 +48,35 @@ class NumericalComponent(Generic[Numerical]):
     lower: Numerical
     upper: Numerical
     length: int = dataclasses.field(init=False, hash=False, compare=False)
+    feature_generator: FeatureGenerator[Numerical] = dataclasses.field(
+        init=False, hash=False, compare=False)
 
     def __post_init__(self):
         object.__setattr__(self, 'length', len(self.possible_values))
+        object.__setattr__(
+            self, 'feature_generator',
+            FeatureGenerator.numerical(
+                name=self.name, lower=self.lower, upper=self.upper))
 
-    def generate(self, index: int) -> Iterator[Dict[str, Any]]:
+    def generate(self, index: int) -> Iterator[
+            Union[Feature[Numerical], Feature[Tuple[Numerical, ...]]]]:
         _index = min(index, self.length - 1)
-        return (
-            {'name': self.name,
-             'categorical': False,
-             'value': vi,
-             'lower': self.lower,
-             'upper': self.upper}
-            for vi in self.possible_values[_index])
+        return (self.feature_generator(vi)
+                for vi in self.possible_values[_index])
 
 
 class ActionGenerator(ReilBase, Generic[Categorical, Numerical]):
     '''
     Gets lists of categorical or numerical lists as components, and generates
-    lists of `ReilData` objects using the product of these components.
+    lists of `FeatureArray` objects using the product of these components.
     '''
 
     def __init__(
             self,
-            components: Optional[
-                Dict[str,
-                     Union[CategoricalComponent, NumericalComponent]]] = None,
+            components: Optional[Dict[
+                str, Union[
+                    CategoricalComponent[Categorical],
+                    NumericalComponent[Numerical]]]] = None,
             **kwargs: Any) -> None:
         '''
         Initializes the `ActionGenerator` instance.
@@ -81,14 +87,17 @@ class ActionGenerator(ReilBase, Generic[Categorical, Numerical]):
             self._components = components
         else:
             self._components: Dict[
-                str, Union[CategoricalComponent, NumericalComponent]] = {}
+                str, Union[
+                    CategoricalComponent[Categorical],
+                    NumericalComponent[Numerical]]] = {}
         self._max_index: int = 0
         self.reset()
 
-    def add_categorical(self,
-                        component_name: str,
-                        possible_values: Tuple[Tuple[Categorical, ...], ...],
-                        categories: Tuple[Categorical, ...]) -> None:
+    def add_categorical(
+            self,
+            component_name: str,
+            possible_values: Tuple[Tuple[Categorical, ...], ...],
+            categories: Tuple[Categorical, ...]) -> None:
         '''
         Add a categorical component.
 
@@ -126,11 +135,12 @@ class ActionGenerator(ReilBase, Generic[Categorical, Numerical]):
             self._max_index,
             len(self._components[component_name].possible_values))
 
-    def add_numerical(self,
-                      component_name: str,
-                      possible_values: Tuple[Tuple[Numerical, ...]],
-                      lower: Numerical,
-                      upper: Numerical) -> None:
+    def add_numerical(
+            self,
+            component_name: str,
+            possible_values: Tuple[Tuple[Numerical, ...]],
+            lower: Numerical,
+            upper: Numerical) -> None:
         '''
         Add a numerical component.
 
@@ -172,15 +182,15 @@ class ActionGenerator(ReilBase, Generic[Categorical, Numerical]):
 
     def possible_actions(
             self,
-            state: Optional[ReilData] = None) -> Tuple[ReilData, ...]:
+            state: Optional[FeatureArray] = None) -> Tuple[FeatureArray, ...]:
         '''
         Generate and return a list of possible actions.
 
         In this implementation, an `index` keeps track of where on the list it
         is on each component, and each call of this method generates the
         product of component values and returns the result as a list of
-        `ReilData`. The `index` is incremented by 1 unit. The last list of a
-        component is used if it is exhausted.
+        `FeatureArray`. The `index` is incremented by 1 unit. The last list of
+        a component is used if it is exhausted.
 
         Arguments
         ---------
@@ -228,7 +238,7 @@ class ActionGenerator(ReilBase, Generic[Categorical, Numerical]):
                 *[component.generate(self._index)
                   for component in self._components.values()])
             self._index += 1
-            result = self._recent_possible_actions = tuple(ReilData(a)
+            result = self._recent_possible_actions = tuple(FeatureArray(a)
                                                            for a in actions)
 
         return result
@@ -239,37 +249,38 @@ class ActionGenerator(ReilBase, Generic[Categorical, Numerical]):
 
     @property
     def lower(self) -> Dict[str, Numerical]:
-        return dict((component.name, component.lower)  # type: ignore
-                    for component in self._components.values()
-                    if hasattr(component, 'lower'))
+        return {component.name: component.lower  # type: ignore
+                for component in self._components.values()
+                if hasattr(component, 'lower')}
 
     @property
     def upper(self) -> Dict[str, Numerical]:
-        return dict((component.name, component.upper)  # type: ignore
-                    for component in self._components.values()
-                    if hasattr(component, 'upper'))
+        return {component.name: component.upper  # type: ignore
+                for component in self._components.values()
+                if hasattr(component, 'upper')}
 
     @property
     def categories(self) -> Dict[str, Tuple[Categorical, ...]]:
-        return dict((component.name, component.categories)  # type: ignore
-                    for component in self._components.values()
-                    if hasattr(component, 'categories'))
+        return {component.name: component.categories  # type: ignore
+                for component in self._components.values()
+                if hasattr(component, 'categories')}
 
     def reset(self) -> None:
         ''' Resets the generator.'''
         self._index: int = 0
-        self._recent_possible_actions: Tuple[ReilData, ...] = ()
+        self._recent_possible_actions: Tuple[FeatureArray, ...] = ()
 
 
 if __name__ == "__main__":
-    AG = ActionGenerator()
+    AG = ActionGenerator[str, int]()
     AG.add_categorical(
         component_name='compass_directions',
         possible_values=(('N',), ('E', 'W'), ('N', 'S')),
         categories=('N', 'S', 'E', 'W'))
-    AG.add_numerical(component_name='odds',
-                     possible_values=((1,), (1, 3)),  # type: ignore
-                     lower=1, upper=9)
+    AG.add_numerical(
+        component_name='odds',
+        possible_values=((1,), (1, 3)),  # type: ignore
+        lower=1, upper=9)
     for i in ('1st', '2nd', '3rd', '4th'):
         print(f'calling possible_actions for the {i} time:')
         for action in AG.possible_actions():
