@@ -9,12 +9,14 @@ an iterator.
 
 from __future__ import annotations
 
+import os
 import pathlib
-from typing import Any, Callable, Generic, Tuple, TypeVar, Union
+from typing import Any, Generic, Tuple, TypeVar, Union
 
-from reil import reilbase
+from reil import reilbase, stateful
+from reil.datatypes.mock_statistic import MockStatistic
 
-T = TypeVar('T', bound=reilbase.ReilBase)
+T = TypeVar('T', bound=stateful.Stateful)
 
 
 class InstanceGenerator(Generic[T], reilbase.ReilBase):
@@ -26,17 +28,18 @@ class InstanceGenerator(Generic[T], reilbase.ReilBase):
     numbers where the instance generator should stop.
     '''
 
-    def __init__(self,
-                 object: T,
-                 instance_counter_stops: Tuple[int] = (-1,),  # -1: infinite
-                 first_instance_number: int = 0,
-                 auto_rewind: bool = False,
-                 save_instances: bool = False,
-                 overwrite_instances: bool = False,
-                 use_existing_instances: bool = True,
-                 save_path: Union[pathlib.Path, str] = '',
-                 filename_pattern: str = '{n:04}',
-                 **kwargs: Any):
+    def __init__(
+            self,
+            object: T,
+            instance_counter_stops: Tuple[int] = (-1,),  # -1: infinite
+            first_instance_number: int = 0,
+            auto_rewind: bool = False,
+            save_instances: bool = False,
+            overwrite_instances: bool = False,
+            use_existing_instances: bool = True,
+            save_path: Union[pathlib.PurePath, str] = '',
+            filename_pattern: str = '{n:04}',
+            **kwargs: Any):
         '''
         Attributes
         ----------
@@ -85,21 +88,25 @@ class InstanceGenerator(Generic[T], reilbase.ReilBase):
         self._last_stop_index = len(self._instance_counter_stops) - 1
 
         self.is_finite = -1 not in instance_counter_stops and not auto_rewind
+        self.statistic = MockStatistic(self._object)
 
         self.rewind()
 
+        if self._object is None:
+            self._logger.info('Instance Generator initialized with None.')
+        else:
+            self._object._name = self._filename_pattern.format(
+                n=self._instance_counter)
+
+    @classmethod
+    def _empty_instance(cls):
+        return cls(None)
+
     def __iter__(self):
         return self
 
-    def __next__(self) -> Tuple[int, T]:  # noqa: C901
-        try:
-            end = self._instance_counter_stops[self._stops_index]
-        except IndexError:
-            if self._auto_rewind:
-                self.rewind()
-                end = self._instance_counter_stops[self._stops_index]
-            else:
-                raise StopIteration
+    def __next__(self) -> Tuple[int, T]:
+        end = self._get_end()
 
         self._instance_counter += 1
 
@@ -108,36 +115,76 @@ class InstanceGenerator(Generic[T], reilbase.ReilBase):
             self._stops_index += 1
 
             if self._stops_index <= self._last_stop_index:
-                if self._instance_counter_stops[self._stops_index] == -1:
-                    self._stop_check: Callable[
-                        [int, int], bool] = lambda current, end: False
-                else:
-                    self._stop_check: Callable[
-                        [int, int], bool] = lambda current, end: current > end
-            raise StopIteration
-        else:
-            current_instance = self._filename_pattern.format(
-                n=self._instance_counter)
-            new_instance = True
-            if self._use_existing_instances:
-                try:
-                    self._object.load(path=self._save_path,
-                                      filename=current_instance)
-                    new_instance = False
-                except FileNotFoundError:
-                    self._object.reset()
-            else:
-                self._object.reset()
+                self._determine_stop_check()
 
-            if self._save_instances and new_instance:
-                if self._overwrite_instances:
-                    self._object.save(path=self._save_path,
+            self._partially_terminated = True
+            raise StopIteration
+
+        else:
+            self._partially_terminated = False
+            self._generate_new_instance()
+
+        return (self._instance_counter, self._object)
+
+    def _get_end(self):
+        try:
+            end = self._instance_counter_stops[self._stops_index]
+        except IndexError:
+            if self._auto_rewind:
+                self.rewind()
+                end = self._instance_counter_stops[self._stops_index]
+            else:
+                raise StopIteration
+        return end
+
+    @staticmethod
+    def _do_not_stop(current: int, end: int) -> bool:
+        return False
+
+    @staticmethod
+    def _do_stop(current: int, end: int) -> bool:
+        return current >= end
+
+    def _determine_stop_check(self):
+        if self._instance_counter_stops[self._stops_index] == -1:
+            self._stop_check = self._do_not_stop
+        else:
+            self._stop_check = self._do_stop
+
+    def _generate_new_instance(self):
+        current_instance = self._filename_pattern.format(
+                n=self._instance_counter)
+        new_instance = True
+        if self._use_existing_instances:
+            try:
+                self._object.load(path=self._save_path,
+                                  filename=current_instance)
+                new_instance = False
+            except FileNotFoundError:
+                self._object.reset()
+        else:
+            self._object.reset()
+
+        self._object._name = current_instance
+        self.statistic.set_object(self._object)
+
+        if self._save_instances and new_instance:
+            if (not self._overwrite_instances and
+                    os.path.isfile(pathlib.Path(
+                        self._save_path, f'{current_instance}.pkl'))):
+                if self._use_existing_instances:
+                    # This extra test is useful when multiple instances
+                    # generators are active. One might create the instance and
+                    # save it, while others are trying to create it. In such
+                    # case, other generators raise exception, but should not!
+                    self._object.load(path=self._save_path,
                                       filename=current_instance)
                 else:
                     raise FileExistsError(
-                        f'File {current_instance} already exists.')
-
-        return (self._instance_counter, self._object)
+                            f'File {current_instance} already exists.')
+            else:
+                self._object.save(path=self._save_path,
+                                  filename=current_instance)
 
     def rewind(self) -> None:
         '''
@@ -145,15 +192,15 @@ class InstanceGenerator(Generic[T], reilbase.ReilBase):
         '''
         self._instance_counter = self._first_instance_number - 1
         self._stops_index = 0
-        if self._instance_counter_stops[self._stops_index] == -1:
-            self._stop_check: Callable[
-                [int, int], bool] = lambda current, end: False
-        else:
-            self._stop_check: Callable[
-                [int, int], bool] = lambda current, end: current > end
+        self._partially_terminated = False
+        self._determine_stop_check()
 
-    def is_terminated(self) -> bool:
-        return self._stops_index > self._last_stop_index
+    def is_terminated(self, fully: bool = True) -> bool:
+        if fully:
+            return not self._auto_rewind and (self._stops_index >
+                                              self._last_stop_index)
+        else:
+            return self._partially_terminated
 
     def __repr__(self) -> str:
         try:
