@@ -9,7 +9,7 @@ This class creates a board for players to play the mnk game.
 import math
 from typing import Any, Iterator, List, Optional, Tuple, Union, overload
 
-from reil.datatypes import ReilData
+from reil.datatypes.feature import FeatureArray, FeatureGenerator
 
 
 class MNKBoard:
@@ -43,6 +43,8 @@ class MNKBoard:
         self._k = k
         self._players = players
         self._can_recapture = can_recapture
+        self._board_state_gen = FeatureGenerator.numerical(
+            name='state', lower=0, upper=self._players)
         self.reset()
 
     @overload
@@ -95,9 +97,10 @@ class MNKBoard:
         '''
         ...
 
-    def set_piece(self, player: int, index: Optional[int] = None,
-                  row: Optional[int] = None, column: Optional[int] = None
-                  ) -> None:
+    def set_piece(  # type: ignore
+            self, player: int, index: Optional[int] = None,
+            row: Optional[int] = None, column: Optional[int] = None
+            ) -> None:
         '''
         Set a piece for a player.
 
@@ -182,9 +185,10 @@ class MNKBoard:
         '''
         ...
 
-    def clear_square(self, index: Optional[int] = None,
-                     row: Optional[int] = None, column: Optional[int] = None
-                     ) -> None:
+    def clear_square(  # type: ignore
+            self, index: Optional[int] = None,
+            row: Optional[int] = None, column: Optional[int] = None
+            ) -> None:
         '''
         Set a piece for a player.
 
@@ -223,10 +227,8 @@ class MNKBoard:
 
     @property
     def board_state(self):
-        ''' Return the state of the board as a ReilData.'''
-        return ReilData.single_numerical(
-            name='state', value=tuple(self._board),
-            lower=0, upper=self._players)
+        ''' Return the state of the board as a FeatureArray.'''
+        return FeatureArray(self._board_state_gen(tuple(self._board)))
 
     def get_board(self,
                   as_list: bool = True) -> Union[List[int], List[List[int]]]:
@@ -293,7 +295,7 @@ class MNKBoard:
             The number of columns. If omitted, it will be infered from the
             `board` size and `m`.
         '''
-        _n = math.ceil(len(board) / m) if n is None else n
+        _n = n or math.ceil(len(board) / m)
 
         return [board[row*_n:(row+1)*_n] for row in range(m)]
 
